@@ -1,12 +1,12 @@
-;;; triples.el --- A flexible triple-based database for us in apps.  -*- lexical-binding: t; -*-
+;;; triples.el --- A flexible triple-based database for use in apps  -*- lexical-binding: t; -*-
 
 ;; Copyright (c) 2022  Free Software Foundation, Inc.
 
 ;; Author: Andrew Hyatt <ahyatt@gmail.com>
 ;; Homepage: https://github.com/ahyatt/triples
-;; Package-Requires: ((seq "2.0") (emacs "25"))
+;; Package-Requires: ((seq "2.0") (emacs "28.1"))
 ;; Keywords: triples, kg, data, sqlite
-;; Version: 0.2.1
+;; Version: 0.2.5
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU General Public License as
 ;; published by the Free Software Foundation; either version 2 of the
@@ -105,21 +105,25 @@ work, therefore this function must be called instead.
 Th DB argument is currently unused, but may be used in the future
 if emacs's native sqlite gains a backup feature.
 
+FILENAME can be nil, if so `triples-default-database-filename'
+will be used.
+
 This also will clear excess backup files, according to
 NUM-TO-KEEP, which specifies how many backup files at max should
 exist at any time. Older backups are the ones that are deleted."
-  (call-process (pcase triples-sqlite-interface
-                  ('builtin triples-sqlite-executable)
-                  ('emacsql emacsql-sqlite-executable))
-                nil nil nil (expand-file-name filename)
-                (format ".backup '%s'" (expand-file-name
-                                        (car (find-backup-file-name
-                                              (expand-file-name filename))))))
-  (let ((backup-files (file-backup-file-names (expand-file-name filename))))
-    (cl-loop for backup-file in (cl-subseq
-                                 backup-files
-                                 (min num-to-keep (length backup-files)))
-             do (delete-file backup-file))))
+  (let ((filename (expand-file-name (or filename triples-default-database-filename))))
+    (call-process (pcase triples-sqlite-interface
+                    ('builtin triples-sqlite-executable)
+                    ('emacsql emacsql-sqlite-executable))
+                  nil nil nil filename
+                  (format ".backup '%s'" (expand-file-name
+                                          (car (find-backup-file-name
+                                                filename)))))
+    (let ((backup-files (file-backup-file-names filename)))
+      (cl-loop for backup-file in (cl-subseq
+                                   backup-files
+                                   (min num-to-keep (length backup-files)))
+               do (delete-file backup-file)))))
 
 (defun triples--decolon (sym)
   "Remove colon from SYM."
@@ -134,7 +138,9 @@ exist at any time. Older backups are the ones that are deleted."
 This is done to have compatibility with the way emacsql stores
 values. Turn a symbol into a string as well, but not a quoted
 one, because sqlite cannot handle symbols."
-  (let ((print-escape-control-characters t))
+  ;; Do not print control characters escaped - we want to get things out exactly
+  ;; as we put them in.
+  (let ((print-escape-control-characters nil))
     (if val
         (format "%S" val)
       ;; Just to save a bit of space, let's use "()" instead of "null", which is
@@ -146,13 +152,9 @@ one, because sqlite cannot handle symbols."
 This imitates the way emacsql returns items, with strings
 becoming either symbols, lists, or strings depending on whether
 the string itself is wrapped in quotes."
-  (if (and (stringp result)
-           (string-prefix-p "\"" result)
-           (string-suffix-p "\"" result))
-      (string-remove-suffix "\"" (string-remove-prefix "\"" result))
-    (if (numberp result)
+  (if (numberp result)
         result
-      (read result))))
+      (read result)))
 
 (defun triples-db-insert (db subject predicate object &optional properties)
   "Insert triple to DB: SUBJECT, PREDICATE, OBJECT with PROPERTIES.
@@ -165,7 +167,7 @@ normal schema checks, so should not be called from client programs."
   (when (and (fboundp 'plistp) (not (plistp properties)))
     (error "Properties stored must always be plists"))
   (pcase triples-sqlite-interface
-    ('builtin 
+    ('builtin
      (sqlite-execute db "REPLACE INTO triples VALUES (?, ?, ?, ?)"
                      (list (triples-standardize-val subject)
                            (triples-standardize-val (triples--decolon predicate))
@@ -220,7 +222,7 @@ all to nil, everything will be deleted, so be careful!"
               (apply #'vector
                      (append '(:delete :from triples)
                              (when (or subject predicate object properties)
-                               (triples--emacsql-andify 
+                               (triples--emacsql-andify
                                 (append
                                  '(:where)
                                  (when subject `((= subject ,(intern (format "$s%d" (cl-incf n))))))
@@ -291,7 +293,7 @@ object, properties to retrieve or nil for *."
                                ,(if selector (apply #'vector selector) '*)
                                :from triples)
                              (when (or subject predicate object properties)
-                               (triples--emacsql-andify 
+                               (triples--emacsql-andify
                                 (append
                                  '(:where)
                                  (when subject `((= subject ,(intern (format "$s%d" (cl-incf n))))))
@@ -441,16 +443,17 @@ The transaction will abort if an error is thrown."
 
 (defun triples--with-transaction (db body-fun)
   (pcase triples-sqlite-interface
-    ('builtin  (condition-case nil
-                   (progn
-                     (sqlite-transaction db)
-                     (funcall body-fun)
-                     (sqlite-commit db))
-                 (error (sqlite-rollback db))))
-    ('emacsql (funcall (triples--eval-when-fboundp emacsql-with-transaction
-                         (lambda (db body-fun)
-                           (emacsql-with-transaction db (funcall body-fun))))
-                       db body-fun))))
+      ('builtin  (condition-case err
+                     (progn
+                       (sqlite-transaction db)
+                       (funcall body-fun)
+                       (sqlite-commit db))
+                   (error (sqlite-rollback db)
+                          (signal (car err) (cdr err)))))
+      ('emacsql (funcall (triples--eval-when-fboundp emacsql-with-transaction
+                           (lambda (db body-fun)
+                             (emacsql-with-transaction db (funcall body-fun))))
+                         db body-fun))))
 
 (defun triples-set-types (db subject &rest combined-props)
   "Set all data for types in COMBINED-PROPS in DB for SUBJECT.
@@ -547,7 +550,9 @@ TYPE-VALS-CONS is a list of conses, combining a type and a plist of values."
           type-vals-cons)))
 
 (defun triples-delete-subject (db subject)
-  "Delete all data in DB associated with SUBJECT."
+  "Delete all data in DB associated with SUBJECT.
+This usually should not be called, it's better to just delete
+data you own with `triples-remove-type'."
   (triples-db-delete db subject))
 
 (defun triples-search (db cpred text)
